<?php
declare(strict_types=1);

namespace FiloBlu\Core\Model;

use FiloBlu\Core\Api\Data\FbOrderExtensionAttrInterfaceFactory;
use FiloBlu\Core\Api\Data\FbOrderExtensionAttrSearchResultsInterfaceFactory;
use FiloBlu\Core\Api\FbOrderExtensionAttrRepositoryInterface;
use FiloBlu\Core\Model\ResourceModel\FbOrderExtensionAttr as ResourceFbOrderExtensionAttr;
use FiloBlu\Core\Model\ResourceModel\FbOrderExtensionAttr\CollectionFactory as FbOrderExtensionAttrCollectionFactory;
use Magento\Framework\Api\DataObjectHelper;
use Magento\Framework\Api\ExtensibleDataObjectConverter;
use Magento\Framework\Api\ExtensionAttribute\JoinProcessorInterface;
use Magento\Framework\Api\SearchCriteria\CollectionProcessorInterface;
use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Framework\Exception\CouldNotDeleteException;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Reflection\DataObjectProcessor;
use Magento\Store\Model\StoreManagerInterface;

use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\Api\FilterBuilder;
use FiloBlu\Core\Api\Data\FbOrderExtensionAttrInterface;

class FbOrderExtensionAttrRepository implements FbOrderExtensionAttrRepositoryInterface
{

    protected $resource;

    protected $extensibleDataObjectConverter;
    protected $searchResultsFactory;

    protected $fbOrderExtensionAttrFactory;

    private $storeManager;

    protected $dataObjectHelper;

    protected $fbOrderExtensionAttrCollectionFactory;

    protected $dataObjectProcessor;

    protected $extensionAttributesJoinProcessor;

    private $collectionProcessor;

    protected $dataFbOrderExtensionAttrFactory;

    protected $searchCriteriaBuilder;
    protected $filterBuilder;


    /**
     * @param ResourceFbOrderExtensionAttr $resource
     * @param FbOrderExtensionAttrFactory $fbOrderExtensionAttrFactory
     * @param FbOrderExtensionAttrInterfaceFactory $dataFbOrderExtensionAttrFactory
     * @param FbOrderExtensionAttrCollectionFactory $fbOrderExtensionAttrCollectionFactory
     * @param FbOrderExtensionAttrSearchResultsInterfaceFactory $searchResultsFactory
     * @param DataObjectHelper $dataObjectHelper
     * @param DataObjectProcessor $dataObjectProcessor
     * @param StoreManagerInterface $storeManager
     * @param CollectionProcessorInterface $collectionProcessor
     * @param JoinProcessorInterface $extensionAttributesJoinProcessor
     * @param ExtensibleDataObjectConverter $extensibleDataObjectConverter
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     * @param FilterBuilder $filterBuilder
     */
    public function __construct(
        ResourceFbOrderExtensionAttr $resource,
        FbOrderExtensionAttrFactory $fbOrderExtensionAttrFactory,
        FbOrderExtensionAttrInterfaceFactory $dataFbOrderExtensionAttrFactory,
        FbOrderExtensionAttrCollectionFactory $fbOrderExtensionAttrCollectionFactory,
        FbOrderExtensionAttrSearchResultsInterfaceFactory $searchResultsFactory,
        DataObjectHelper $dataObjectHelper,
        DataObjectProcessor $dataObjectProcessor,
        StoreManagerInterface $storeManager,
        CollectionProcessorInterface $collectionProcessor,
        JoinProcessorInterface $extensionAttributesJoinProcessor,
        ExtensibleDataObjectConverter $extensibleDataObjectConverter,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        FilterBuilder $filterBuilder
    ) {
        $this->resource = $resource;
        $this->fbOrderExtensionAttrFactory = $fbOrderExtensionAttrFactory;
        $this->fbOrderExtensionAttrCollectionFactory = $fbOrderExtensionAttrCollectionFactory;
        $this->searchResultsFactory = $searchResultsFactory;
        $this->dataObjectHelper = $dataObjectHelper;
        $this->dataFbOrderExtensionAttrFactory = $dataFbOrderExtensionAttrFactory;
        $this->dataObjectProcessor = $dataObjectProcessor;
        $this->storeManager = $storeManager;
        $this->collectionProcessor = $collectionProcessor;
        $this->extensionAttributesJoinProcessor = $extensionAttributesJoinProcessor;
        $this->extensibleDataObjectConverter = $extensibleDataObjectConverter;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->filterBuilder = $filterBuilder;
    }

    /**
     * {@inheritdoc}
     */
    public function save(
        FbOrderExtensionAttrInterface $fbOrderExtensionAttr
    ) {
        /* if (empty($fbOrderExtensionAttr->getStoreId())) {
            $storeId = $this->storeManager->getStore()->getId();
            $fbOrderExtensionAttr->setStoreId($storeId);
        } */

        $fbOrderExtensionAttrData = $this->extensibleDataObjectConverter->toNestedArray(
            $fbOrderExtensionAttr,
            [],
            FbOrderExtensionAttrInterface::class
        );

        $fbOrderExtensionAttrModel = $this->fbOrderExtensionAttrFactory->create()->setData($fbOrderExtensionAttrData);

        try {
            $this->resource->save($fbOrderExtensionAttrModel);
        } catch (\Exception $exception) {
            throw new CouldNotSaveException(__(
                'Could not save the fbOrderExtensionAttr: %1',
                $exception->getMessage()
            ));
        }
        return $fbOrderExtensionAttrModel->getDataModel();
    }

    /**
     * {@inheritdoc}
     */
    public function get($fbOrderExtensionAttrId)
    {
        $fbOrderExtensionAttr = $this->fbOrderExtensionAttrFactory->create();
        $this->resource->load($fbOrderExtensionAttr, $fbOrderExtensionAttrId);
        if (!$fbOrderExtensionAttr->getId()) {
            throw new NoSuchEntityException(__('FbOrderExtensionAttr with id "%1" does not exist.', $fbOrderExtensionAttrId));
        }
        return $fbOrderExtensionAttr->getDataModel();
    }

    /**
     * {@inheritdoc}
     */
    public function getByOrderId($order_id){

        $fbOrderExtensionAttr = $this->fbOrderExtensionAttrFactory->create();
        $this->resource->load($fbOrderExtensionAttr, $order_id,FbOrderExtensionAttrInterface::ORDER_ID);
        if (!$fbOrderExtensionAttr->getId()) {
            throw new NoSuchEntityException(__('FbOrderExtensionAttr with order_id "%1" does not exist.', $order_id));
        }
        return $fbOrderExtensionAttr->getDataModel();
    }

    /**
     * {@inheritdoc}
     */
    public function getList(
        SearchCriteriaInterface $criteria
    ) {
        $collection = $this->fbOrderExtensionAttrCollectionFactory->create();

        $this->extensionAttributesJoinProcessor->process(
            $collection,
            FbOrderExtensionAttrInterface::class
        );

        $this->collectionProcessor->process($criteria, $collection);

        $searchResults = $this->searchResultsFactory->create();
        $searchResults->setSearchCriteria($criteria);

        $items = [];
        foreach ($collection as $model) {
            $items[] = $model->getDataModel();
        }

        $searchResults->setItems($items);
        $searchResults->setTotalCount($collection->getSize());
        return $searchResults;
    }

    /**
     * {@inheritdoc}
     */
    public function delete(
        FbOrderExtensionAttrInterface $fbOrderExtensionAttr
    ) {
        try {
            $fbOrderExtensionAttrModel = $this->fbOrderExtensionAttrFactory->create();
            $this->resource->load($fbOrderExtensionAttrModel, $fbOrderExtensionAttr->getFborderextensionattrId());
            $this->resource->delete($fbOrderExtensionAttrModel);
        } catch (\Exception $exception) {
            throw new CouldNotDeleteException(__(
                'Could not delete the FbOrderExtensionAttr: %1',
                $exception->getMessage()
            ));
        }
        return true;
    }

    /**
     * {@inheritdoc}
     */
    public function deleteById($fbOrderExtensionAttrId)
    {
        return $this->delete($this->get($fbOrderExtensionAttrId));
    }
}

